////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Qualcomm Technologies, Inc. and/or its subsidiaries.
// All Rights Reserved.
// Confidential and Proprietary - Qualcomm Technologies, Inc.
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// THIS IS AUTO-GENERATED CODE.  PLEASE DO NOT EDIT (File bug reports against tools).
///
/// Auto-generated by: Parameter Parser V5.5.1 (2411131018)
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @file  modscconvv1.cpp
/// @brief Auto-generated Chromatix parameter file
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "modscconvv1.h"
#include "parametersetmanager.h"

MODSCCONVV1_NAMESPACE_BEGIN

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// SetValue/GetValue Functions
/// Gets or sets the requested value
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool modSmartSCDThrDataType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "triggerPt") == 0)
    {
        __result = triggerPt.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "triggerData") == 0)
    {
        if (__index < triggerDataCount)
        {
            __result = triggerData[__index].GetValue(&__path[__pos], __value, __size);
        }
    }

    return __result;
}

bool modSmartSCDThrDataType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "triggerPt") == 0)
    {
        __result = triggerPt.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "triggerData") == 0)
    {
        if (__index < triggerDataCount)
        {
            __result = triggerData[__index].SetValue(&__path[__pos], __value);
        }
    }

    return __result;
}

bool modSmartSCDThrDataType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // triggerPt
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("triggerPt", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "triggerPt") == 0)
    {
        __result = triggerPt.GetInfo(&__path[__pos], __info);
    }

    // triggerData
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("triggerData", ParameterParserDataType::Structure, true, false, triggerDataCount));
    }
    else if (PARAMETER_STRCMP(__name, "triggerData") == 0)
    {
        if (__index < triggerDataCount)
        {
            __result = triggerData[__index].GetInfo(&__path[__pos], __info);
        }
        else
        {
            __result = true;
            __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Structure, true, false, triggerDataCount));
        }
    }

    return __result;
}

bool FIRFilterConfigType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "samples") == 0)
    {
        ParameterUtils::DoubleToString((double)samples, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "coefficient") == 0)
    {
        if (__index < coefficientCount)
        {
            ParameterUtils::DoubleToString((double)coefficient[__index], __value, __size);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < coefficientCount; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)coefficient[i], &__value[__valuePos], (INT32)(__size - __valuePos));
            }
            __result = true;
        }
    }

    return __result;
}

bool FIRFilterConfigType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "samples") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            samples = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "coefficient") == 0)
    {
        if (__index < coefficientCount)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                coefficient[__index] = (float)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < coefficientCount)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    coefficient[__index] = (float)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }

    return __result;
}

bool FIRFilterConfigType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // samples
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("samples", ParameterParserDataType::Int32, (double)samples, 0));
    }
    else if (PARAMETER_STRCMP(__name, "samples") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)samples, 0));
    }

    // coefficient
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("coefficient", ParameterParserDataType::Float, true, false, coefficientCount));
    }

    return __result;
}

bool MedianFilterConfigType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "samples") == 0)
    {
        ParameterUtils::DoubleToString((double)samples, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool MedianFilterConfigType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "samples") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            samples = (INT32)dblValue; 
        }
    }

    return __result;
}

bool MedianFilterConfigType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // samples
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("samples", ParameterParserDataType::Int32, (double)samples, 0));
    }
    else if (PARAMETER_STRCMP(__name, "samples") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)samples, 0));
    }

    return __result;
}

bool IIRFilterConfigType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "samplesInput") == 0)
    {
        ParameterUtils::DoubleToString((double)samplesInput, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "coeffInput") == 0)
    {
        if (__index < coeffInputCount)
        {
            ParameterUtils::DoubleToString((double)coeffInput[__index], __value, __size);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < coeffInputCount; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)coeffInput[i], &__value[__valuePos], (INT32)(__size - __valuePos));
            }
            __result = true;
        }
    }
    else if (PARAMETER_STRCMP(__name, "samplesOutput") == 0)
    {
        ParameterUtils::DoubleToString((double)samplesOutput, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "coeffOutput") == 0)
    {
        if (__index < coeffOutputCount)
        {
            ParameterUtils::DoubleToString((double)coeffOutput[__index], __value, __size);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < coeffOutputCount; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)coeffOutput[i], &__value[__valuePos], (INT32)(__size - __valuePos));
            }
            __result = true;
        }
    }

    return __result;
}

bool IIRFilterConfigType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "samplesInput") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            samplesInput = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "coeffInput") == 0)
    {
        if (__index < coeffInputCount)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                coeffInput[__index] = (float)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < coeffInputCount)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    coeffInput[__index] = (float)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }
    else if (PARAMETER_STRCMP(__name, "samplesOutput") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            samplesOutput = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "coeffOutput") == 0)
    {
        if (__index < coeffOutputCount)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                coeffOutput[__index] = (float)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < coeffOutputCount)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    coeffOutput[__index] = (float)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }

    return __result;
}

bool IIRFilterConfigType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // samplesInput
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("samplesInput", ParameterParserDataType::Int32, (double)samplesInput, 0));
    }
    else if (PARAMETER_STRCMP(__name, "samplesInput") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)samplesInput, 0));
    }

    // coeffInput
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("coeffInput", ParameterParserDataType::Float, true, false, coeffInputCount));
    }

    // samplesOutput
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("samplesOutput", ParameterParserDataType::Int32, (double)samplesOutput, 0));
    }
    else if (PARAMETER_STRCMP(__name, "samplesOutput") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)samplesOutput, 0));
    }

    // coeffOutput
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("coeffOutput", ParameterParserDataType::Float, true, false, coeffOutputCount));
    }

    return __result;
}

bool FilterConfigType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "type") == 0)
    {
        ParameterUtils::DoubleToString((double)(UINT32)type, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "fir") == 0)
    {
        __result = fir.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "iir") == 0)
    {
        __result = iir.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "median") == 0)
    {
        __result = median.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool FilterConfigType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "type") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            type = (awbFilterType)(UINT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "fir") == 0)
    {
        __result = fir.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "iir") == 0)
    {
        __result = iir.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "median") == 0)
    {
        __result = median.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool FilterConfigType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // type
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("type", ParameterParserDataType::Enum, (double)(UINT32)type, 0));
    }
    else if (PARAMETER_STRCMP(__name, "type") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Enum, (double)(UINT32)type, 0));
    }

    // fir
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("fir", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "fir") == 0)
    {
        __result = fir.GetInfo(&__path[__pos], __info);
    }

    // iir
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("iir", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "iir") == 0)
    {
        __result = iir.GetInfo(&__path[__pos], __info);
    }

    // median
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("median", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "median") == 0)
    {
        __result = median.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool SCDetectorConfigType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "type") == 0)
    {
        ParameterUtils::DoubleToString((double)(UINT32)type, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "fixedBaselineEn") == 0)
    {
        ParameterUtils::DoubleToString((double)fixedBaselineEn, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "fixedBaselineValue") == 0)
    {
        ParameterUtils::DoubleToString((double)fixedBaselineValue, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "triggerTh") == 0)
    {
        ParameterUtils::DoubleToString((double)triggerTh, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "consCountTh") == 0)
    {
        ParameterUtils::DoubleToString((double)consCountTh, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "postSCDFramesTh") == 0)
    {
        ParameterUtils::DoubleToString((double)postSCDFramesTh, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "postSCDStableTh") == 0)
    {
        ParameterUtils::DoubleToString((double)postSCDStableTh, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "holdTime") == 0)
    {
        ParameterUtils::DoubleToString((double)holdTime, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "strictHoldTime") == 0)
    {
        ParameterUtils::DoubleToString((double)strictHoldTime, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "postSCDStableTypeAbs") == 0)
    {
        ParameterUtils::DoubleToString((double)postSCDStableTypeAbs, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool SCDetectorConfigType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "type") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            type = (awbSCDTriggerType)(UINT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "fixedBaselineEn") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            fixedBaselineEn = (BOOL)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "fixedBaselineValue") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            fixedBaselineValue = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "triggerTh") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            triggerTh = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "consCountTh") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            consCountTh = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "postSCDFramesTh") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            postSCDFramesTh = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "postSCDStableTh") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            postSCDStableTh = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "holdTime") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            holdTime = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "strictHoldTime") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            strictHoldTime = (BOOL)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "postSCDStableTypeAbs") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            postSCDStableTypeAbs = (BOOL)dblValue; 
        }
    }

    return __result;
}

bool SCDetectorConfigType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // type
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("type", ParameterParserDataType::Enum, (double)(UINT32)type, 0));
    }
    else if (PARAMETER_STRCMP(__name, "type") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Enum, (double)(UINT32)type, 0));
    }

    // fixedBaselineEn
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("fixedBaselineEn", ParameterParserDataType::Boolean, (double)fixedBaselineEn, 0));
    }
    else if (PARAMETER_STRCMP(__name, "fixedBaselineEn") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Boolean, (double)fixedBaselineEn, 0));
    }

    // fixedBaselineValue
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("fixedBaselineValue", ParameterParserDataType::Float, (double)fixedBaselineValue));
    }
    else if (PARAMETER_STRCMP(__name, "fixedBaselineValue") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)fixedBaselineValue));
    }

    // triggerTh
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("triggerTh", ParameterParserDataType::Float, (double)triggerTh));
    }
    else if (PARAMETER_STRCMP(__name, "triggerTh") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)triggerTh));
    }

    // consCountTh
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("consCountTh", ParameterParserDataType::Int32, (double)consCountTh, 0));
    }
    else if (PARAMETER_STRCMP(__name, "consCountTh") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)consCountTh, 0));
    }

    // postSCDFramesTh
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("postSCDFramesTh", ParameterParserDataType::Int32, (double)postSCDFramesTh, 0));
    }
    else if (PARAMETER_STRCMP(__name, "postSCDFramesTh") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)postSCDFramesTh, 0));
    }

    // postSCDStableTh
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("postSCDStableTh", ParameterParserDataType::Float, (double)postSCDStableTh));
    }
    else if (PARAMETER_STRCMP(__name, "postSCDStableTh") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)postSCDStableTh));
    }

    // holdTime
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("holdTime", ParameterParserDataType::Float, (double)holdTime));
    }
    else if (PARAMETER_STRCMP(__name, "holdTime") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)holdTime));
    }

    // strictHoldTime
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("strictHoldTime", ParameterParserDataType::Boolean, (double)strictHoldTime, 0));
    }
    else if (PARAMETER_STRCMP(__name, "strictHoldTime") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Boolean, (double)strictHoldTime, 0));
    }

    // postSCDStableTypeAbs
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("postSCDStableTypeAbs", ParameterParserDataType::Boolean, (double)postSCDStableTypeAbs, 0));
    }
    else if (PARAMETER_STRCMP(__name, "postSCDStableTypeAbs") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Boolean, (double)postSCDStableTypeAbs, 0));
    }

    return __result;
}

bool FilterDetectorConfigType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enable") == 0)
    {
        ParameterUtils::DoubleToString((double)enable, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "filter") == 0)
    {
        __result = filter.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "detector") == 0)
    {
        __result = detector.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool FilterDetectorConfigType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enable") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            enable = (BOOL)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "filter") == 0)
    {
        __result = filter.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "detector") == 0)
    {
        __result = detector.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool FilterDetectorConfigType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // enable
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("enable", ParameterParserDataType::Boolean, (double)enable, 0));
    }
    else if (PARAMETER_STRCMP(__name, "enable") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Boolean, (double)enable, 0));
    }

    // filter
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("filter", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "filter") == 0)
    {
        __result = filter.GetInfo(&__path[__pos], __info);
    }

    // detector
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("detector", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "detector") == 0)
    {
        __result = detector.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool SceneChangeDetectorConfigType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "Aggregate") == 0)
    {
        ParameterUtils::DoubleToString((double)(UINT32)Aggregate, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "filterDetector") == 0)
    {
        if (__index < filterDetectorCount)
        {
            __result = filterDetector[__index].GetValue(&__path[__pos], __value, __size);
        }
    }

    return __result;
}

bool SceneChangeDetectorConfigType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "Aggregate") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            Aggregate = (awbAggregateType)(UINT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "filterDetector") == 0)
    {
        if (__index < filterDetectorCount)
        {
            __result = filterDetector[__index].SetValue(&__path[__pos], __value);
        }
    }

    return __result;
}

bool SceneChangeDetectorConfigType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // Aggregate
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("Aggregate", ParameterParserDataType::Enum, (double)(UINT32)Aggregate, 0));
    }
    else if (PARAMETER_STRCMP(__name, "Aggregate") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Enum, (double)(UINT32)Aggregate, 0));
    }

    // filterDetector
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("filterDetector", ParameterParserDataType::Structure, true, false, filterDetectorCount));
    }
    else if (PARAMETER_STRCMP(__name, "filterDetector") == 0)
    {
        if (__index < filterDetectorCount)
        {
            __result = filterDetector[__index].GetInfo(&__path[__pos], __info);
        }
        else
        {
            __result = true;
            __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Structure, true, false, filterDetectorCount));
        }
    }

    return __result;
}

bool SCConvType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "slowConvReduceFactor") == 0)
    {
        ParameterUtils::DoubleToString((double)slowConvReduceFactor, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "fastConvBoostFactor") == 0)
    {
        ParameterUtils::DoubleToString((double)fastConvBoostFactor, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "fastConvergePercTh") == 0)
    {
        ParameterUtils::DoubleToString((double)fastConvergePercTh, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "waitFramesBeforeSCStart") == 0)
    {
        ParameterUtils::DoubleToString((double)waitFramesBeforeSCStart, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool SCConvType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "slowConvReduceFactor") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            slowConvReduceFactor = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "fastConvBoostFactor") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            fastConvBoostFactor = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "fastConvergePercTh") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            fastConvergePercTh = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "waitFramesBeforeSCStart") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            waitFramesBeforeSCStart = (INT32)dblValue; 
        }
    }

    return __result;
}

bool SCConvType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // slowConvReduceFactor
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("slowConvReduceFactor", ParameterParserDataType::Float, (double)slowConvReduceFactor));
    }
    else if (PARAMETER_STRCMP(__name, "slowConvReduceFactor") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)slowConvReduceFactor));
    }

    // fastConvBoostFactor
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("fastConvBoostFactor", ParameterParserDataType::Float, (double)fastConvBoostFactor));
    }
    else if (PARAMETER_STRCMP(__name, "fastConvBoostFactor") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)fastConvBoostFactor));
    }

    // fastConvergePercTh
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("fastConvergePercTh", ParameterParserDataType::Float, (double)fastConvergePercTh));
    }
    else if (PARAMETER_STRCMP(__name, "fastConvergePercTh") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)fastConvergePercTh));
    }

    // waitFramesBeforeSCStart
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("waitFramesBeforeSCStart", ParameterParserDataType::Int32, (double)waitFramesBeforeSCStart, 0));
    }
    else if (PARAMETER_STRCMP(__name, "waitFramesBeforeSCStart") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)waitFramesBeforeSCStart, 0));
    }

    return __result;
}

bool SmartSCDType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "SmartSCDEn") == 0)
    {
        ParameterUtils::DoubleToString((double)SmartSCDEn, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "SmartSCDSampling") == 0)
    {
        ParameterUtils::DoubleToString((double)SmartSCDSampling, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "DeltaEColorChangeThr") == 0)
    {
        ParameterUtils::DoubleToString((double)DeltaEColorChangeThr, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "TempFoVChangeRatio") == 0)
    {
        ParameterUtils::DoubleToString((double)TempFoVChangeRatio, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "FinalSmartSCDThreshold") == 0)
    {
        __result = FinalSmartSCDThreshold.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool SmartSCDType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "SmartSCDEn") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            SmartSCDEn = (BOOL)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "SmartSCDSampling") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            SmartSCDSampling = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "DeltaEColorChangeThr") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            DeltaEColorChangeThr = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "TempFoVChangeRatio") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            TempFoVChangeRatio = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "FinalSmartSCDThreshold") == 0)
    {
        __result = FinalSmartSCDThreshold.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool SmartSCDType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // SmartSCDEn
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("SmartSCDEn", ParameterParserDataType::Boolean, (double)SmartSCDEn, 0));
    }
    else if (PARAMETER_STRCMP(__name, "SmartSCDEn") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Boolean, (double)SmartSCDEn, 0));
    }

    // SmartSCDSampling
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("SmartSCDSampling", ParameterParserDataType::Int32, (double)SmartSCDSampling, 0));
    }
    else if (PARAMETER_STRCMP(__name, "SmartSCDSampling") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)SmartSCDSampling, 0));
    }

    // DeltaEColorChangeThr
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("DeltaEColorChangeThr", ParameterParserDataType::Float, (double)DeltaEColorChangeThr));
    }
    else if (PARAMETER_STRCMP(__name, "DeltaEColorChangeThr") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)DeltaEColorChangeThr));
    }

    // TempFoVChangeRatio
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("TempFoVChangeRatio", ParameterParserDataType::Float, (double)TempFoVChangeRatio));
    }
    else if (PARAMETER_STRCMP(__name, "TempFoVChangeRatio") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)TempFoVChangeRatio));
    }

    // FinalSmartSCDThreshold
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("FinalSmartSCDThreshold", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "FinalSmartSCDThreshold") == 0)
    {
        __result = FinalSmartSCDThreshold.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool chromatixSCConvV1Type::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "moduleID") == 0)
    {
        __result = moduleID.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "module_version") == 0)
    {
        __result = module_version.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "enable") == 0)
    {
        ParameterUtils::DoubleToString((double)enable, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "detector") == 0)
    {
        __result = detector.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "converge") == 0)
    {
        __result = converge.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "smartconverge") == 0)
    {
        __result = smartconverge.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool chromatixSCConvV1Type::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "moduleID") == 0)
    {
        __result = moduleID.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "module_version") == 0)
    {
        __result = module_version.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "enable") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            enable = (BOOL)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "detector") == 0)
    {
        __result = detector.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "converge") == 0)
    {
        __result = converge.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "smartconverge") == 0)
    {
        __result = smartconverge.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool chromatixSCConvV1Type::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // moduleID
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("moduleID", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "moduleID") == 0)
    {
        __result = moduleID.GetInfo(&__path[__pos], __info);
    }

    // module_version
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("module_version", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "module_version") == 0)
    {
        __result = module_version.GetInfo(&__path[__pos], __info);
    }

    // enable
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("enable", ParameterParserDataType::Boolean, (double)enable, 0));
    }
    else if (PARAMETER_STRCMP(__name, "enable") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Boolean, (double)enable, 0));
    }

    // detector
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("detector", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "detector") == 0)
    {
        __result = detector.GetInfo(&__path[__pos], __info);
    }

    // converge
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("converge", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "converge") == 0)
    {
        __result = converge.GetInfo(&__path[__pos], __info);
    }

    // smartconverge
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("smartconverge", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "smartconverge") == 0)
    {
        __result = smartconverge.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool chromatixSCConvV1Type::module_versionStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "major_revision") == 0)
    {
        ParameterUtils::DoubleToString((double)major_revision, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "minor_revision") == 0)
    {
        ParameterUtils::DoubleToString((double)minor_revision, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "incr_revision") == 0)
    {
        ParameterUtils::DoubleToString((double)incr_revision, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool chromatixSCConvV1Type::module_versionStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "major_revision") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            major_revision = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "minor_revision") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            minor_revision = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "incr_revision") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            incr_revision = (INT32)dblValue; 
        }
    }

    return __result;
}

bool chromatixSCConvV1Type::module_versionStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // major_revision
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("major_revision", ParameterParserDataType::Int32, (double)major_revision, 0));
    }
    else if (PARAMETER_STRCMP(__name, "major_revision") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)major_revision, 0));
    }

    // minor_revision
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("minor_revision", ParameterParserDataType::Int32, (double)minor_revision, 0));
    }
    else if (PARAMETER_STRCMP(__name, "minor_revision") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)minor_revision, 0));
    }

    // incr_revision
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("incr_revision", ParameterParserDataType::Int32, (double)incr_revision, 0));
    }
    else if (PARAMETER_STRCMP(__name, "incr_revision") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)incr_revision, 0));
    }

    return __result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @brief Read enum value
/// @param entry [in] Symbol table entry
/// @param enumValue [out] Enum value to read
/// @param alignment [in] Symbol table byte alignment
/// @return true if successful
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool ReadEnum_awbFilterType(
    ParameterFileSymbolTableEntry* const entry,
    awbFilterType& enumValue,
    BYTE const alignment)
{
    bool result {false};

    UINT32 uintValue {entry->ReadEnum(&result, alignment)};

    if (result)
    {
        switch (uintValue)
        {
        case static_cast<UINT32>(awbFilterType::NoFilter):
            enumValue = awbFilterType::NoFilter;
            result    = true;
            break;
        case static_cast<UINT32>(awbFilterType::FIRFilter):
            enumValue = awbFilterType::FIRFilter;
            result    = true;
            break;
        case static_cast<UINT32>(awbFilterType::medianFilter):
            enumValue = awbFilterType::medianFilter;
            result    = true;
            break;
        case static_cast<UINT32>(awbFilterType::IIRFilter):
            enumValue = awbFilterType::IIRFilter;
            result    = true;
            break;
        case static_cast<UINT32>(awbFilterType::MinMaxDiffFilter):
            enumValue = awbFilterType::MinMaxDiffFilter;
            result    = true;
            break;
        default:
            PARAMETER_LOG_ERROR(entry->SetManager, "Invalid value(% u) converting UINT32 to awbFilterType", uintValue);
            break;
        }
    }
    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @brief Read enum value
/// @param entry [in] Symbol table entry
/// @param enumValue [out] Enum value to read
/// @param alignment [in] Symbol table byte alignment
/// @return true if successful
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool ReadEnum_awbSCDTriggerType(
    ParameterFileSymbolTableEntry* const entry,
    awbSCDTriggerType& enumValue,
    BYTE const alignment)
{
    bool result {false};

    UINT32 uintValue {entry->ReadEnum(&result, alignment)};

    if (result)
    {
        switch (uintValue)
        {
        case static_cast<UINT32>(awbSCDTriggerType::PercGain):
            enumValue = awbSCDTriggerType::PercGain;
            result    = true;
            break;
        case static_cast<UINT32>(awbSCDTriggerType::CCT):
            enumValue = awbSCDTriggerType::CCT;
            result    = true;
            break;
        case static_cast<UINT32>(awbSCDTriggerType::Lux):
            enumValue = awbSCDTriggerType::Lux;
            result    = true;
            break;
        case static_cast<UINT32>(awbSCDTriggerType::Motion):
            enumValue = awbSCDTriggerType::Motion;
            result    = true;
            break;
        case static_cast<UINT32>(awbSCDTriggerType::Gyro):
            enumValue = awbSCDTriggerType::Gyro;
            result    = true;
            break;
        case static_cast<UINT32>(awbSCDTriggerType::Res1):
            enumValue = awbSCDTriggerType::Res1;
            result    = true;
            break;
        case static_cast<UINT32>(awbSCDTriggerType::Res2):
            enumValue = awbSCDTriggerType::Res2;
            result    = true;
            break;
        default:
            PARAMETER_LOG_ERROR(entry->SetManager, "Invalid value(% u) converting UINT32 to awbSCDTriggerType", uintValue);
            break;
        }
    }
    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @brief Read enum value
/// @param entry [in] Symbol table entry
/// @param enumValue [out] Enum value to read
/// @param alignment [in] Symbol table byte alignment
/// @return true if successful
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool ReadEnum_awbAggregateType(
    ParameterFileSymbolTableEntry* const entry,
    awbAggregateType& enumValue,
    BYTE const alignment)
{
    bool result {false};

    UINT32 uintValue {entry->ReadEnum(&result, alignment)};

    if (result)
    {
        switch (uintValue)
        {
        case static_cast<UINT32>(awbAggregateType::SCBasedOnAnyOne):
            enumValue = awbAggregateType::SCBasedOnAnyOne;
            result    = true;
            break;
        case static_cast<UINT32>(awbAggregateType::SCBasedOnAll):
            enumValue = awbAggregateType::SCBasedOnAll;
            result    = true;
            break;
        case static_cast<UINT32>(awbAggregateType::Res1):
            enumValue = awbAggregateType::Res1;
            result    = true;
            break;
        case static_cast<UINT32>(awbAggregateType::Res2):
            enumValue = awbAggregateType::Res2;
            result    = true;
            break;
        default:
            PARAMETER_LOG_ERROR(entry->SetManager, "Invalid value(% u) converting UINT32 to awbAggregateType", uintValue);
            break;
        }
    }
    return result;
}


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Default Constructor
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
chromatixSCConvV1TypeClass::chromatixSCConvV1TypeClass(
    const CHAR* const name)
    : ParameterModule(name, MAJOR, MINOR, PATCH, nullptr)
{
    PARAMETER_INITIALIZE(SymbolTableID);
    PARAMETER_INITIALIZE(moduleID);
    PARAMETER_INITIALIZE(module_version);
    PARAMETER_INITIALIZE(enable);
    PARAMETER_INITIALIZE(detector);
    PARAMETER_INITIALIZE(converge);
    PARAMETER_INITIALIZE(smartconverge);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Constructor
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
chromatixSCConvV1TypeClass::chromatixSCConvV1TypeClass(
    const CHAR* const name,
    ModuleData* const moduleData)
    : ParameterModule(name, MAJOR, MINOR, PATCH, moduleData)
{
    PARAMETER_INITIALIZE(SymbolTableID);
    PARAMETER_INITIALIZE(moduleID);
    PARAMETER_INITIALIZE(module_version);
    PARAMETER_INITIALIZE(enable);
    PARAMETER_INITIALIZE(detector);
    PARAMETER_INITIALIZE(converge);
    PARAMETER_INITIALIZE(smartconverge);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Destructor
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
chromatixSCConvV1TypeClass::~chromatixSCConvV1TypeClass()
{
    if (Loaded)
    {
        UnloadModule();
    }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// GetName
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
const CHAR* chromatixSCConvV1TypeClass::GetName(
    const CHAR* const type)
{
    const CHAR* name = type;
    CHAR temp[ParameterFileSymbolTableEntry::TYPE_LEN + 1];
    if (PARAMETER_STRCMP(type, ModuleData::GetType(temp, "awbSceneChangeConvergeV1")) == 0)
    {
        name = "awbSceneChangeConvergeV1";
    }
    if (PARAMETER_STRCMP(type, ModuleData::GetType(temp, "scConv")) == 0)
    {
        name = "scConv";
    }
    return name;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load modSmartSCDThrDataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool modSmartSCDThrDataType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(triggerPt);
    PARAMETER_INITIALIZE(triggerData);

    result = triggerPt.Load(entry, alignment);

    // Reading Array triggerData from symbol table
    // Do not check result value here because the count value needs to be initialized in this block
    {
        triggerDataCount = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, triggerDataCount, alignment)
        constexpr const CHAR* const file {__FILE__};
        constexpr INT32 line {LINE_CONST};
        ParameterFileSymbolTableEntry  pointerEntryCls;
        ParameterFileSymbolTableEntry* const pointerEntry = &pointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(pointerEntryCls, entry, alignment);
        }

        if (result)
        {
            triggerDataID = pointerEntry->ID;
            DupObjInfo* triggerDataDupInfo = pointerEntry->GetDupObject(triggerDataCount, file, line);
            if (triggerDataDupInfo != nullptr && triggerDataDupInfo->Object != nullptr)
            {
                triggerData = (awbglobalelements::awbTriggerDataRGBType*)triggerDataDupInfo->Object;
                if (triggerData == nullptr)
                {
                    if (triggerDataDupInfo->File != nullptr)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "triggerData duplicate found but object is nullptr: %s:%d, RefCount: %d\n",
                            triggerDataDupInfo->File, triggerDataDupInfo->Line, triggerDataDupInfo->RefCount);
                    }
                    else
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "triggerData duplicate found but Object is nullptr: <nullptr file ptr>:%d, RefCount: %d\n",
                            triggerDataDupInfo->Line, triggerDataDupInfo->RefCount);
                    }
                }
            }

            if (triggerData == nullptr)
            {
                if (result && triggerDataCount > 0)
                {
                    triggerData = PARAMETER_NEW awbglobalelements::awbTriggerDataRGBType[triggerDataCount];
                    (void)pointerEntry->SetDupObject(triggerData, triggerDataCount, file, line);
                }
                if (!result || triggerData == nullptr)
                {
                    if (triggerDataCount > 0)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "Failed to allocate triggerData\n");
                    }
                    triggerDataCount = 0;
                }
                for (UINT32 i = 0; result && i < triggerDataCount; i++)
                {
                    result = triggerData[i].Load(pointerEntry, alignment);
                }
            }
        }
        if (triggerData == nullptr)
        {
            triggerDataCount = 0;
        }
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load FIRFilterConfigType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool FIRFilterConfigType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(samples);
    PARAMETER_INITIALIZE(coefficient);

    SYMBOL_TABLE_ENTRY_READ(entry, result, samples, alignment)

    // Reading Array coefficient from symbol table
    // Do not check result value here because the count value needs to be initialized in this block
    {
        coefficientCount = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, coefficientCount, alignment)
        constexpr const CHAR* const file {__FILE__};
        constexpr INT32 line {LINE_CONST};
        ParameterFileSymbolTableEntry  pointerEntryCls;
        ParameterFileSymbolTableEntry* const pointerEntry = &pointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(pointerEntryCls, entry, alignment);
        }

        if (result)
        {
            coefficientID = pointerEntry->ID;
            DupObjInfo* coefficientDupInfo = pointerEntry->GetDupObject(coefficientCount, file, line);
            if (coefficientDupInfo != nullptr && coefficientDupInfo->Object != nullptr)
            {
                coefficient = (float*)coefficientDupInfo->Object;
                if (coefficient == nullptr)
                {
                    if (coefficientDupInfo->File != nullptr)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "coefficient duplicate found but object is nullptr: %s:%d, RefCount: %d\n",
                            coefficientDupInfo->File, coefficientDupInfo->Line, coefficientDupInfo->RefCount);
                    }
                    else
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "coefficient duplicate found but Object is nullptr: <nullptr file ptr>:%d, RefCount: %d\n",
                            coefficientDupInfo->Line, coefficientDupInfo->RefCount);
                    }
                }
            }

            if (coefficient == nullptr)
            {
                if (result && coefficientCount > 0)
                {
                    coefficient = pointerEntry->ReadArray<float>(coefficientCount, alignment);
                    result = coefficient != nullptr;
                }
                if (!result || coefficient == nullptr)
                {
                    if (coefficientCount > 0)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "Failed to allocate coefficient\n");
                    }
                    coefficientCount = 0;
                }
            }
        }
        if (coefficient == nullptr)
        {
            coefficientCount = 0;
        }
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load MedianFilterConfigType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool MedianFilterConfigType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(samples);

    SYMBOL_TABLE_ENTRY_READ(entry, result, samples, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load IIRFilterConfigType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool IIRFilterConfigType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(samplesInput);
    PARAMETER_INITIALIZE(coeffInput);
    PARAMETER_INITIALIZE(samplesOutput);
    PARAMETER_INITIALIZE(coeffOutput);

    SYMBOL_TABLE_ENTRY_READ(entry, result, samplesInput, alignment)

    // Reading Array coeffInput from symbol table
    // Do not check result value here because the count value needs to be initialized in this block
    {
        coeffInputCount = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, coeffInputCount, alignment)
        constexpr const CHAR* const file {__FILE__};
        constexpr INT32 line {LINE_CONST};
        ParameterFileSymbolTableEntry  pointerEntryCls;
        ParameterFileSymbolTableEntry* const pointerEntry = &pointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(pointerEntryCls, entry, alignment);
        }

        if (result)
        {
            coeffInputID = pointerEntry->ID;
            DupObjInfo* coeffInputDupInfo = pointerEntry->GetDupObject(coeffInputCount, file, line);
            if (coeffInputDupInfo != nullptr && coeffInputDupInfo->Object != nullptr)
            {
                coeffInput = (float*)coeffInputDupInfo->Object;
                if (coeffInput == nullptr)
                {
                    if (coeffInputDupInfo->File != nullptr)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "coeffInput duplicate found but object is nullptr: %s:%d, RefCount: %d\n",
                            coeffInputDupInfo->File, coeffInputDupInfo->Line, coeffInputDupInfo->RefCount);
                    }
                    else
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "coeffInput duplicate found but Object is nullptr: <nullptr file ptr>:%d, RefCount: %d\n",
                            coeffInputDupInfo->Line, coeffInputDupInfo->RefCount);
                    }
                }
            }

            if (coeffInput == nullptr)
            {
                if (result && coeffInputCount > 0)
                {
                    coeffInput = pointerEntry->ReadArray<float>(coeffInputCount, alignment);
                    result = coeffInput != nullptr;
                }
                if (!result || coeffInput == nullptr)
                {
                    if (coeffInputCount > 0)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "Failed to allocate coeffInput\n");
                    }
                    coeffInputCount = 0;
                }
            }
        }
        if (coeffInput == nullptr)
        {
            coeffInputCount = 0;
        }
    }
    SYMBOL_TABLE_ENTRY_READ(entry, result, samplesOutput, alignment)

    // Reading Array coeffOutput from symbol table
    // Do not check result value here because the count value needs to be initialized in this block
    {
        coeffOutputCount = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, coeffOutputCount, alignment)
        constexpr const CHAR* const file {__FILE__};
        constexpr INT32 line {LINE_CONST};
        ParameterFileSymbolTableEntry  pointerEntryCls;
        ParameterFileSymbolTableEntry* const pointerEntry = &pointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(pointerEntryCls, entry, alignment);
        }

        if (result)
        {
            coeffOutputID = pointerEntry->ID;
            DupObjInfo* coeffOutputDupInfo = pointerEntry->GetDupObject(coeffOutputCount, file, line);
            if (coeffOutputDupInfo != nullptr && coeffOutputDupInfo->Object != nullptr)
            {
                coeffOutput = (float*)coeffOutputDupInfo->Object;
                if (coeffOutput == nullptr)
                {
                    if (coeffOutputDupInfo->File != nullptr)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "coeffOutput duplicate found but object is nullptr: %s:%d, RefCount: %d\n",
                            coeffOutputDupInfo->File, coeffOutputDupInfo->Line, coeffOutputDupInfo->RefCount);
                    }
                    else
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "coeffOutput duplicate found but Object is nullptr: <nullptr file ptr>:%d, RefCount: %d\n",
                            coeffOutputDupInfo->Line, coeffOutputDupInfo->RefCount);
                    }
                }
            }

            if (coeffOutput == nullptr)
            {
                if (result && coeffOutputCount > 0)
                {
                    coeffOutput = pointerEntry->ReadArray<float>(coeffOutputCount, alignment);
                    result = coeffOutput != nullptr;
                }
                if (!result || coeffOutput == nullptr)
                {
                    if (coeffOutputCount > 0)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "Failed to allocate coeffOutput\n");
                    }
                    coeffOutputCount = 0;
                }
            }
        }
        if (coeffOutput == nullptr)
        {
            coeffOutputCount = 0;
        }
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load FilterConfigType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool FilterConfigType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(type);
    PARAMETER_INITIALIZE(fir);
    PARAMETER_INITIALIZE(iir);
    PARAMETER_INITIALIZE(median);

    result = modscconvv1::ReadEnum_awbFilterType(entry, type, alignment);
    if (result)
    {
        result = fir.Load(entry, alignment);
    }
    if (result)
    {
        result = iir.Load(entry, alignment);
    }
    if (result)
    {
        result = median.Load(entry, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load SCDetectorConfigType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool SCDetectorConfigType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(type);
    PARAMETER_INITIALIZE(fixedBaselineEn);
    PARAMETER_INITIALIZE(fixedBaselineValue);
    PARAMETER_INITIALIZE(triggerTh);
    PARAMETER_INITIALIZE(consCountTh);
    PARAMETER_INITIALIZE(postSCDFramesTh);
    PARAMETER_INITIALIZE(postSCDStableTh);
    PARAMETER_INITIALIZE(holdTime);
    PARAMETER_INITIALIZE(strictHoldTime);
    PARAMETER_INITIALIZE(postSCDStableTypeAbs);

    result = modscconvv1::ReadEnum_awbSCDTriggerType(entry, type, alignment);
    SYMBOL_TABLE_ENTRY_READ(entry, result, fixedBaselineEn, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, fixedBaselineValue, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, triggerTh, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, consCountTh, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, postSCDFramesTh, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, postSCDStableTh, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, holdTime, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, strictHoldTime, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, postSCDStableTypeAbs, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load FilterDetectorConfigType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool FilterDetectorConfigType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(enable);
    PARAMETER_INITIALIZE(filter);
    PARAMETER_INITIALIZE(detector);

    SYMBOL_TABLE_ENTRY_READ(entry, result, enable, alignment)
    if (result)
    {
        result = filter.Load(entry, alignment);
    }
    if (result)
    {
        result = detector.Load(entry, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load SceneChangeDetectorConfigType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool SceneChangeDetectorConfigType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(Aggregate);
    PARAMETER_INITIALIZE(filterDetector);

    result = modscconvv1::ReadEnum_awbAggregateType(entry, Aggregate, alignment);

    // Reading Array filterDetector from symbol table
    // Do not check result value here because the count value needs to be initialized in this block
    {
        filterDetectorCount = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, filterDetectorCount, alignment)
        constexpr const CHAR* const file {__FILE__};
        constexpr INT32 line {LINE_CONST};
        ParameterFileSymbolTableEntry  pointerEntryCls;
        ParameterFileSymbolTableEntry* const pointerEntry = &pointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(pointerEntryCls, entry, alignment);
        }

        if (result)
        {
            filterDetectorID = pointerEntry->ID;
            DupObjInfo* filterDetectorDupInfo = pointerEntry->GetDupObject(filterDetectorCount, file, line);
            if (filterDetectorDupInfo != nullptr && filterDetectorDupInfo->Object != nullptr)
            {
                filterDetector = (FilterDetectorConfigType*)filterDetectorDupInfo->Object;
                if (filterDetector == nullptr)
                {
                    if (filterDetectorDupInfo->File != nullptr)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "filterDetector duplicate found but object is nullptr: %s:%d, RefCount: %d\n",
                            filterDetectorDupInfo->File, filterDetectorDupInfo->Line, filterDetectorDupInfo->RefCount);
                    }
                    else
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "filterDetector duplicate found but Object is nullptr: <nullptr file ptr>:%d, RefCount: %d\n",
                            filterDetectorDupInfo->Line, filterDetectorDupInfo->RefCount);
                    }
                }
            }

            if (filterDetector == nullptr)
            {
                if (result && filterDetectorCount > 0)
                {
                    filterDetector = PARAMETER_NEW FilterDetectorConfigType[filterDetectorCount];
                    (void)pointerEntry->SetDupObject(filterDetector, filterDetectorCount, file, line);
                }
                if (!result || filterDetector == nullptr)
                {
                    if (filterDetectorCount > 0)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "Failed to allocate filterDetector\n");
                    }
                    filterDetectorCount = 0;
                }
                for (UINT32 i = 0; result && i < filterDetectorCount; i++)
                {
                    result = filterDetector[i].Load(pointerEntry, alignment);
                }
            }
        }
        if (filterDetector == nullptr)
        {
            filterDetectorCount = 0;
        }
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load SCConvType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool SCConvType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(slowConvReduceFactor);
    PARAMETER_INITIALIZE(fastConvBoostFactor);
    PARAMETER_INITIALIZE(fastConvergePercTh);
    PARAMETER_INITIALIZE(waitFramesBeforeSCStart);

    SYMBOL_TABLE_ENTRY_READ(entry, result, slowConvReduceFactor, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, fastConvBoostFactor, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, fastConvergePercTh, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, waitFramesBeforeSCStart, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load SmartSCDType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool SmartSCDType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(SmartSCDEn);
    PARAMETER_INITIALIZE(SmartSCDSampling);
    PARAMETER_INITIALIZE(DeltaEColorChangeThr);
    PARAMETER_INITIALIZE(TempFoVChangeRatio);
    PARAMETER_INITIALIZE(FinalSmartSCDThreshold);

    SYMBOL_TABLE_ENTRY_READ(entry, result, SmartSCDEn, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, SmartSCDSampling, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, DeltaEColorChangeThr, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, TempFoVChangeRatio, alignment)
    if (result)
    {
        result = FinalSmartSCDThreshold.Load(entry, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load chromatixSCConvV1Type
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatixSCConvV1Type::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(moduleID);
    PARAMETER_INITIALIZE(module_version);
    PARAMETER_INITIALIZE(enable);
    PARAMETER_INITIALIZE(detector);
    PARAMETER_INITIALIZE(converge);
    PARAMETER_INITIALIZE(smartconverge);

    result = moduleID.Load(entry, alignment);
    if (result)
    {
        result = module_version.Load(entry, alignment);
    }
    SYMBOL_TABLE_ENTRY_READ(entry, result, enable, alignment)
    if (result)
    {
        result = detector.Load(entry, alignment);
    }
    if (result)
    {
        result = converge.Load(entry, alignment);
    }
    if (result)
    {
        result = smartconverge.Load(entry, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load module_versionStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatixSCConvV1Type::module_versionStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(major_revision);
    PARAMETER_INITIALIZE(minor_revision);
    PARAMETER_INITIALIZE(incr_revision);

    SYMBOL_TABLE_ENTRY_READ(entry, result, major_revision, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, minor_revision, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, incr_revision, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload modSmartSCDThrDataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void modSmartSCDThrDataType::Unload(ParameterFileSymbolTable* const __table)
{
    if (__table != nullptr && __table->ReleaseDupObject(triggerData))
    {
        PARAMETER_DELETE[] triggerData;
    }
    triggerData = nullptr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload FIRFilterConfigType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void FIRFilterConfigType::Unload(ParameterFileSymbolTable* const __table)
{
    if (__table != nullptr && __table->ReleaseDupObject(coefficient))
    {
        PARAMETER_DELETE[] coefficient;
    }
    coefficient = nullptr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload IIRFilterConfigType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void IIRFilterConfigType::Unload(ParameterFileSymbolTable* const __table)
{
    if (__table != nullptr && __table->ReleaseDupObject(coeffInput))
    {
        PARAMETER_DELETE[] coeffInput;
    }
    coeffInput = nullptr;
    if (__table != nullptr && __table->ReleaseDupObject(coeffOutput))
    {
        PARAMETER_DELETE[] coeffOutput;
    }
    coeffOutput = nullptr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload FilterConfigType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void FilterConfigType::Unload(ParameterFileSymbolTable* const __table)
{
    fir.Unload(__table);
    iir.Unload(__table);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload FilterDetectorConfigType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void FilterDetectorConfigType::Unload(ParameterFileSymbolTable* const __table)
{
    filter.Unload(__table);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload SceneChangeDetectorConfigType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void SceneChangeDetectorConfigType::Unload(ParameterFileSymbolTable* const __table)
{
    for (UINT32 i = 0; i < filterDetectorCount; i++)
    {
        filterDetector[i].Unload(__table);
    }
    if (__table != nullptr && __table->ReleaseDupObject(filterDetector))
    {
        PARAMETER_DELETE[] filterDetector;
    }
    filterDetector = nullptr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload SmartSCDType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void SmartSCDType::Unload(ParameterFileSymbolTable* const __table)
{
    FinalSmartSCDThreshold.Unload(__table);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload chromatixSCConvV1Type
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatixSCConvV1Type::Unload(ParameterFileSymbolTable* const __table)
{
    detector.Unload(__table);
    smartconverge.Unload(__table);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Release the ParameterModule object
/// Requires that this object was created via a ParameterSetManager::GetModule call
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatixSCConvV1Type::Release()
{
    ((chromatixSCConvV1TypeClass*)this)->Release();
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Returns the ParameterModule::Modified value
/// The Modified value is incremented when the module's data or pointer values change
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
UINT32 chromatixSCConvV1Type::GetModified()
{
    return ((chromatixSCConvV1TypeClass*)this)->Modified;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Create module, and return a new object
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
ParameterModule* chromatixSCConvV1TypeClass::Create(
    ModuleData*    moduleData,
    DefaultModule* const defModule) const
{
    chromatixSCConvV1TypeClass* cls = nullptr;

    if (PARAMETER_STRCMP(Type, moduleData->Type) == 0 && Version.Value == moduleData->Version.Value)
    {
        const UINT32 id = moduleData->DuplicateID > 0 ? moduleData->DuplicateID : moduleData->ID;

        ParameterModule* const module = defModule->Find(id);
        if (module != nullptr)
        {
            cls = (chromatixSCConvV1TypeClass*)module;
        }
        else
        {
            cls = PARAMETER_NEW chromatixSCConvV1TypeClass(GetName(moduleData->Type), moduleData);
            cls->SymbolTableID = id;
            defModule->AddModule((ParameterModule*)cls);
        }
    }
    else if (Version.Value != moduleData->Version.Value)
    {
        PARAMETER_LOG_ERROR(moduleData->SetManager, "Unable to create chromatixSCConvV1TypeClass, binary version (%lld) does not match source version (%lld)\n",
            moduleData->Version.Value, Version.Value);
    }
    else
    {
        PARAMETER_LOG_ERROR(moduleData->SetManager, "Unable to create chromatixSCConvV1TypeClass, binary name (%s) does not match source name (%s)\n",
            moduleData->Type, Type);
    }

    return (ParameterModule*)cls;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load module data, return true is successful
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatixSCConvV1TypeClass::LoadModule(
    ParameterFileSymbolTable* const table)
{
    bool result = true;

    if (!Loaded)
    {
        BYTE const alignment = table->Alignment;
        chromatixSCConvV1TypeClass* const cls = this;

        ParameterFileSymbolTableEntry  entryObj;
        ParameterFileSymbolTableEntry* const entry = &entryObj;
        // With EzTune enabled, the unused upper byte of SymbolTableID value can change, so clear it
        result = table->Find(entryObj, SymbolTableID & 0x00ffffff);

        if (!result)
        {
            PARAMETER_LOG_ERROR(entry->SetManager, "chromatixSCConvV1TypeClass::LoadModule Failed to find module ID (%d) in Symbol Table\n", SymbolTableID);
        }

        if (result)
        {
            result = cls->Load(entry, alignment);
        }
        Modified++;
        Loaded = true;
    }

    if (!result)
    {
        /// Module failed to fully load, unload it
        UnloadModule();
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload module data
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatixSCConvV1TypeClass::UnloadModule()
{
    if (Loaded)
    {
        if (Data != nullptr && Data->SymbolTable != nullptr)
        {
            Unload(Data->SymbolTable);
        }
        else
        {
            PARAMETER_LOG_ERROR(Data != nullptr ? Data->SetManager : nullptr, "UnloadModule %s called with nullptr "
                "Data->SymbolTable object\n", Name);
            Unload(nullptr);
        }
        Loaded = false;
    }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// /// Release the ParameterModule object
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatixSCConvV1TypeClass::Release()
{
    ParameterModule::Release();
}

MODSCCONVV1_NAMESPACE_END
